﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http.Headers;
using System.Reactive.Linq;
using System.Reactive.Threading.Tasks;
using System.Threading;
using System.Threading.Tasks;
using Dicom;
using Dicom.Network.Client;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using NLog;

namespace SendFileWebapi.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class SCUController : ControllerBase
    {

        static Logger logger = LogManager.GetCurrentClassLogger();

        [HttpPost]
        public async Task Post([FromForm]Models.InputsDto inputsDto)
        {
            var client = new DicomClient(inputsDto.ServerHost, inputsDto.Port, false, inputsDto.ServerAET, inputsDto.Aet);
            client.NegotiateAsyncOps();
            inputsDto.FileNames.ToObservable()
                .Select(fileName => Observable.FromAsync(async () =>
            {
                var stream = fileName.OpenReadStream();
                var request = new Dicom.Network.DicomCStoreRequest(await DicomFile.OpenAsync(stream));
                await client.AddRequestAsync(request);
                await client.SendAsync();
                stream.Dispose();
            }).Select(x => fileName.FileName).Catch<string, Exception>(exp => throw new Exception($"failed to send: {fileName}, with error:{exp.Message}")))
                .Concat()
                .Subscribe(
                    filename =>
                        logger.Warn($"file sent:{filename}"),
                    onError =>
                        logger.Error(onError.Message),
                    () => { });
        }

    }
}
